interface IPropertyData {
    classification: 'SystemMetaData' | 'CallstackOrException' | 'CustomerContent' | 'PublicNonPersonalData' | 'EndUserPseudonymizedInformation';
    purpose: 'PerformanceAndHealth' | 'FeatureInsight' | 'BusinessInsight';
    comment: string;
    expiration?: string;
    endpoint?: string;
    isMeasurement?: boolean;
}
interface IGDPRProperty {
    owner: string;
    comment: string;
    expiration?: string;
    readonly [name: string]: IPropertyData | undefined | IGDPRProperty | string;
}
type IGDPRPropertyWithoutMetadata = Omit<IGDPRProperty, 'owner' | 'comment' | 'expiration'>;
type OmitMetadata<T> = Omit<T, 'owner' | 'comment' | 'expiration'>;
type ClassifiedEvent<T extends IGDPRPropertyWithoutMetadata> = {
    [k in keyof T]: any;
};
type StrictPropertyChecker<TEvent, TClassification, TError> = keyof TEvent extends keyof OmitMetadata<TClassification> ? keyof OmitMetadata<TClassification> extends keyof TEvent ? TEvent : TError : TError;
type StrictPropertyCheckError = {
    error: 'Type of classified event does not match event properties';
};
type StrictPropertyCheck<T extends IGDPRProperty, E> = StrictPropertyChecker<E, ClassifiedEvent<OmitMetadata<T>>, StrictPropertyCheckError>;

export type { ClassifiedEvent, IGDPRProperty, IPropertyData, OmitMetadata, StrictPropertyCheck, StrictPropertyCheckError, StrictPropertyChecker };
